; How to Draw an Asymmetric Reflected Playfield
; by Roger Williams
; Reflected mode modifications by Glenn Saunders
; with a lot of help from Nick Bensema
; (this started out as his "How to Draw a Playfield")
;
; Additional modifications, bug-fixes by Wade Brown
;
;Roger Williams:
;
; For comparison, I have tightened up Nick's code
; just a little by eliminating all the subroutine
; calling overhead, which is more like how a real
; game would work, and eliminated the scrolling and
; color effects to isolate them from the pure act of
; displaying a bitmap.
;
;
;Wade Brown:
;
; Added PF_PIX_HEIGHT, PF_MAX_IDX, comments, equates, etc. 
; Fixed off by one Errors.  Can anyone figure out how to get ANY cycles
; after the final write to PF2? Or should I be using Z26?
;
;
	processor 6502
	include vcs.h
;-----------------------------------------------
; Equates
;-----------------------------------------------
T64_VBLANK		EQU	43		; 43 * 64 = 2752 cycles	(vblank= (76 * 37 = 2812 cycles))
T64_OSCAN		EQU	35		; 35 * 64 = 2240 cycles (oscan = (76 * 30 = 2280 cycles))
PF_Reflect		EQU $01		; ** un-comment if not using Glenn Saunders' vcs.h - WB **
SCAN_LINES		EQU 191
PF_PIX_HEIGHT	EQU	8		; each 20 bits of pf data gets drawn this many times
PF_MAX_IDX		EQU 23		; maximum index for pf data (total pfheight = PF_PIX_HEIGHT * (PF_MAX_IDX + 1))
							; Note: This will be the Height from Playfield-gen - 1.
RomStart        EQU $F000
;-----------------------------------------------
; Macros
;-----------------------------------------------
	mac	WAIT_TIMER	
.1		LDA	INTIM	;Wait for the proper scan line
		BNE	.1
	endm
;-----------------------------------------------
; Ram Segment (128 bytes)
;-----------------------------------------------
    SEG.U Vars
    ORG $80       
ScanLine	DS	1			; usual scanline counter
ScanPix		DS	1			; pf pixel counter
;
Temp		DS	1			; just in case...
;-----------------------------------------------
; Code Segment
;-----------------------------------------------
	SEG	Code
	ORG	RomStart
Start
;
	SEI  ; Disable interrupts, if there are any.
	CLD  ; Clear BCD math bit.
	LDX  #$FF
	TXS  ; Set stack to beginning.
	LDA #0
B1      
	STA 0,X
	DEX
	BNE B1
MainLoop
	LDX  #0	   ;vertical blank
	LDA  #2
	STA  WSYNC  
	STA  WSYNC
	STA  WSYNC
	STA  VSYNC ;Begin vertical sync.
	STA  WSYNC ; First line of VSYNC
	STA  WSYNC ; Second line of VSYNC.

	LDA  #44   ;init timer for overscan
	STA  TIM64T

	STA  WSYNC		; Third line of VSYNC.
	STA  VSYNC		; (0)


	LDA #0
	STA COLUBK		; Background will be black.

VBLOOP	LDA INTIM
	BNE VBLOOP		; Whew!
	STA WSYNC
	STA VBLANK		; End the VBLANK period with a zero.
	LDA PF_Reflect
	STA CTRLPF		; set playfield to reflect - G.S.
	LDA #0
	;
	STA COLUP0
	STA COLUP1		; blacking out player colors
	LDA #$34		; set playfield color to deep red
	STA COLUPF
	;
	LDA #191 
	STA ScanLine
	LDA #PF_PIX_HEIGHT			; each pf data element is this many pixels tall
	STA ScanPix
	;
	LDA #0
	STA PF0
	STA PF1
	STA PF2; zero out player shapes
	;
	LDX #PF_MAX_IDX
ScanLoop
	DEC ScanPix				; next screen pixel line
	BPL ScanGo				; Fixed off by one error - Wade Brown
	LDA #PF_PIX_HEIGHT-1
	STA ScanPix				; ScanPix "Wrap-around" - get next pf data element
	DEX						; next big pixel line
	BPL ScanGo				; Fixed off by one error - Wade Brown	
	LDX #PF_MAX_IDX			; pf "wrap-around"
ScanGo
	STA WSYNC

	;six cycles available  Might be able to do something here
	nop			;[21]+2
	nop			;[23]+2
	nop			;[25]+2
	;
	; Time to begin cycle counting
	;
	LDA PFData0,X           ;[0]+4
	STA PF0                 ;[4]+3 = *7*   < 23	;PF0 visible
	
	LDA PFData1,X           ;[7]+4
	STA PF1                 ;[11]+3 = *14*  < 29	;PF1 visible
	
	LDA PFData2,X           ;[14]+4
	STA PF2                 ;[18]+3 = *21*  < 40	;PF2 visible
	;
	; we can eat six cycles here or after WSYNC 
	;	
	LDA PFData0b,X          ;[27]+4

	;PF0 no longer visible, safe to rewrite
	STA PF0                 ;[31]+3 = *34* 

	LDA PFData1b,X		;[34]+4

	;PF1 no longer visible, safe to rewrite
	STA PF1			;[38]+3 = *41*  


	LDA PFData2b,X		;[41]+4
	
	;PF2 rewrite must begin at exactly cycle 45!!, no more, no less
	STA PF2			;[45]+2 = *47*  ; > 46 PF2 no longer visible

	;76-47 = 29 cycles left per scanline available for
	; further calculations (i.e. sprites), not counting the rest of the
	; existing code below that branches off and changes the indexing to the letter graphics

	;nop adding even this one nop, however, breaks this kernel!
	;-Glenn Saunders
	DEC ScanLine
	BNE ScanLoop
	;
	LDA #2
	STA WSYNC  ;Finish scanline 192.
	STA VBLANK ; Make TIA output invisible,
	;----------------------------------------
	; Start Overscan timer
	;----------------------------------------
	LDA	#T64_OSCAN		; total number of cycles for 30 scan-line OverScan period
	STA	TIM64T			;Set the timer for start of screen
	; ----------------------------------------------------
	; We have approx 2240 cycles for logic here (overscan)
	; ----------------------------------------------------

	;----------------------------------------
	; Wait for end of overscan
	;----------------------------------------
	WAIT_TIMER
	JMP  MainLoop      ;Continue forever.
;*******************************************************
;
; Subroutines
;
;*******************************************************



; *********************** 
; GRAPHICS DATA
; *********************** 
GrPage	ALIGN 256 
;               REFLECTED PLAYFIELD
;   PF0|   PF1  |  PF2   |   PF2b |   PF1b |PF0b|
;  4567|76543210|01234567|76543210|01234567|7654|

PFData0	
	.byte %11110000
	.byte %11110000
	.byte %00000000
	.byte %11100000
	.byte %11000000
	.byte %11010000
	.byte %11010000
	.byte %10010000
	.byte %10110000
	.byte %00010000
	.byte %11000000
	.byte %11000000
	.byte %11010000
	.byte %10010000
	.byte %10000000
	.byte %10100000
	.byte %10100000
	.byte %10100000
	.byte %00100000
	.byte %11100000
	.byte %00000000
	.byte %11010000
	.byte %11010000
	.byte %00010000

PFData1	
	.byte %00011100
	.byte %01000000
	.byte %01110110
	.byte %01100111
	.byte %01101111
	.byte %00000000
	.byte %00111111
	.byte %10001111
	.byte %10100110
	.byte %01110000
	.byte %00001111
	.byte %11100000
	.byte %00010101
	.byte %01010101
	.byte %01010101
	.byte %01010101
	.byte %01010101
	.byte %01010000
	.byte %01011111
	.byte %11001111
	.byte %01000001
	.byte %00011101
	.byte %10001100
	.byte %00100001

PFData2	
	.byte %00000101
	.byte %01110101
	.byte %00110000
	.byte %10111101
	.byte %10000001
	.byte %10101000
	.byte %00101011
	.byte %01001000
	.byte %01011010
	.byte %01011011
	.byte %01010011
	.byte %00000110
	.byte %10110110
	.byte %10000110
	.byte %00110000
	.byte %01111011
	.byte %01100000
	.byte %00101110
	.byte %10000010
	.byte %11001010
	.byte %00000000
	.byte %10100100
	.byte %00000010
	.byte %01001000


PFData2b
	.byte %00000000
	.byte %11000000
	.byte %10000000
	.byte %10000000
	.byte %10000000
	.byte %10000000
	.byte %10000000
	.byte %10000000
	.byte %10000001
	.byte %00000001
	.byte %00000001
	.byte %00000001
	.byte %00000001
	.byte %10000001
	.byte %10000001
	.byte %10000001
	.byte %10000000
	.byte %10000000
	.byte %10000000
	.byte %10000000
	.byte %10000000
	.byte %10000000
	.byte %10000000
	.byte %10000000

PFData1b
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000111
	.byte %00000101
	.byte %00001100
	.byte %00001000
	.byte %00001000
	.byte %00000100
	.byte %00000110
	.byte %00000010
	.byte %00000011
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000
	.byte %00000000

PFData0b
	.byte %00010000
	.byte %00010000
	.byte %00110000
	.byte %01100000
	.byte %01000000
	.byte %01000000
	.byte %01100000
	.byte %00100000
	.byte %00110000
	.byte %00010000
	.byte %00010000
	.byte %00010000
	.byte %00010000
	.byte %00010000
	.byte %00010000
	.byte %00010000
	.byte %00010000
	.byte %00110000
	.byte %00100000
	.byte %00100000
	.byte %00100000
	.byte %01100000
	.byte %11000000
	.byte %10000000


	org $FFFC
        .word Start
        .word Start
